@@ -7,7 +7,6 @@ from astropy.time import Time, TimeDelta
 import astropy.units as u
 from functools import partial
 from kafkit.registry.aiohttp import RegistryApi
-import logging
 import pandas as pd
 import requests
 
@@ -24,19 +23,15 @@ class EfdClient:
         Name of the EFD instance for which to retrieve credentials.
     db_name : `str`, optional
         Name of the database within influxDB to query ('efd' by default).
-    port : `str`, optional
-        Port to use when querying the database ('443' by default).
     creds_service : `str`, optional
         URL to the service to retrieve credentials
         (``https://roundtable.lsst.codes/segwarides/`` by default).
+    timeout : `int`, optional
+        Timeout in seconds for async requests (`aiohttp.client`). The default
+        timeout is 900 seconds.
     client : `object`, optional
         An instance of a class that ducktypes as `aioinflux.InfluxDBClient`.
         The intent is to be able to substitute a mocked client for testing.
-    convert_influx_index : `bool`, optional
-        Convert influxDB time index from TAI to UTC?  This is for using legacy
-        instances that may still have timestamps stored internally as TAI.
-        Modern instances all store index timestamps as UTC natively.
-        Default is `False`, don't translate from TAI to UTC.
     """
 
     influx_client = None
@@ -48,16 +43,18 @@ class EfdClient:
     subclasses = {}
     deployment = ''
 
-    def __init__(self, efd_name, db_name='efd', port='443',
+    def __init__(self, efd_name, db_name='efd',
                  creds_service='https://roundtable.lsst.codes/segwarides/',
-                 client=None, convert_influx_index=False):
+                 timeout=900, client=None):
         self.db_name = db_name
         self.auth = NotebookAuth(service_endpoint=creds_service)
-        host, schema_registry, user, password = self.auth.get_auth(efd_name)
-        self.schema_registry = schema_registry
-        self.convert_influx_index = convert_influx_index
+        host, schema_registry, port, user, password = self.auth.get_auth(efd_name)
+        if schema_registry[-1] == '/':
+            self.schema_registry = schema_registry[-1] + ':' + port + '/'
+        else:
+            self.schema_registry = schema_registry + ':' + port
         if client is None:
-            health_url = f'https://{host}/health'
+            health_url = f'https://{host}:{port}/health'
             response = requests.get(health_url)
             if response.status_code != 200:
                 raise RuntimeError(f'InfluxDB server, {host}, does not appear ready to '
@@ -69,8 +66,9 @@ class EfdClient:
                                                           username=user,
                                                           password=password,
                                                           db=db_name,
-                                                          mode='async')  # mode='blocking')
-            self.influx_client.output = 'dataframe'
+                                                          mode='async',
+                                                          output='dataframe',
+                                                          timeout=timeout)
         else:
             self.influx_client = client
         self.query_history = []
@@ -120,13 +118,15 @@ class EfdClient:
             raise NotImplementedError(f'There is no EFD client class implemented for {efd_name}.')
         return self.subclasses[efd_name](efd_name, *args, **kwargs)
 
-    async def _do_query(self, query):
+    async def _do_query(self, query, convert_influx_index=False):
         """Query the influxDB and return results
 
         Parameters
         ----------
         query : `str`
             Query string to execute.
+        convert_influx_index : `bool`
+            Legacy flag to convert time index from TAI to UTC
 
         Returns
         -------
@@ -138,9 +138,9 @@ class EfdClient:
         if not isinstance(result, pd.DataFrame) and not result:
             # aioinflux returns an empty dict for an empty query
             result = pd.DataFrame()
-        elif self.convert_influx_index:
+        elif convert_influx_index:
             times = Time(result.index, format='datetime', scale='tai')
-            result = result.set_index(times.utc.datetime) 
+            result = result.set_index(times.utc.datetime)
         return result
 
     async def get_topics(self):
@@ -170,7 +170,9 @@ class EfdClient:
         fields = await self._do_query(f'SHOW FIELD KEYS FROM "{self.db_name}"."autogen"."{topic_name}"')
         return fields['fieldKey'].tolist()
 
-    def build_time_range_query(self, topic_name, fields, start, end, is_window=False, index=None):
+    def build_time_range_query(self, topic_name, fields, start, end, is_window=False,
+                               index=None, convert_influx_index=False,
+                               use_old_csc_indexing=False):
         """Build a query based on a time range.
 
         Parameters
@@ -190,8 +192,17 @@ class EfdClient:
             `~astropy.time.TimeDelta`, compute a range centered on the start
             time (default is `False`).
         index : `int`, optional
-            For indexed topics set this to the index of the topic to query
+            When index is used, add an 'AND salIndex = index' to the query.
             (default is `None`).
+        convert_influx_index : `bool`, optional
+            Convert influxDB time index from TAI to UTC?  This is for using legacy
+            instances that may still have timestamps stored internally as TAI.
+            Modern instances all store index timestamps as UTC natively.
+            Default is `False`, don't translate from TAI to UTC.
+        use_old_csc_indexing: `bool`, optional
+            When index is used, add an 'AND {CSCName}ID = index' to the query
+            which is the old CSC indexing name.
+            (default is `False`).
 
         Returns
         -------
@@ -202,27 +213,36 @@ class EfdClient:
             raise TypeError('The first time argument must be a time stamp')
 
         if not start.scale == 'utc':
-            raise ValueError(f'Timestamps must be in UTC.')
+            raise ValueError('Timestamps must be in UTC.')
+
+        if convert_influx_index:  # Implies index is in TAI, so query should be in TAI
+            start = start.tai
 
         if isinstance(end, TimeDelta):
             if is_window:
-                start_str = (start - end/2).isot
-                end_str = (start + end/2).isot
+                start_str = (start - end / 2).isot
+                end_str = (start + end / 2).isot
             else:
                 start_str = start.isot
                 end_str = (start + end).isot
         elif isinstance(end, Time):
             if not end.scale == 'utc':
-                raise ValueError(f'Timestamps must be in UTC.')
+                raise ValueError('Timestamps must be in UTC.')
+            if convert_influx_index:
+                end = end.tai
             start_str = start.isot
             end_str = end.isot
         else:
-            raise TypeError('The second time argument must be the time stamp for the end ' +
+            raise TypeError('The second time argument must be the time stamp for the end '
                             'or a time delta.')
         index_str = ''
         if index:
-            parts = topic_name.split('.')
-            index_str = f' AND {parts[-2]}ID = {index}'  # The CSC name is always the penultimate
+            if use_old_csc_indexing:
+                parts = topic_name.split('.')
+                index_name = f'{parts[-2]}ID'  # The CSC name is always the penultimate
+            else:
+                index_name = 'salIndex'
+            index_str = f' AND {index_name} = {index}'
         timespan = f"time >= '{start_str}Z' AND time <= '{end_str}Z'{index_str}"  # influxdb demands last Z
 
         if isinstance(fields, str):
@@ -234,7 +254,9 @@ class EfdClient:
         # Build query here
         return f'SELECT {", ".join(fields)} FROM "{self.db_name}"."autogen"."{topic_name}" WHERE {timespan}'
 
-    async def select_time_series(self, topic_name, fields, start, end, is_window=False, index=None):
+    async def select_time_series(self, topic_name, fields, start, end, is_window=False,
+                                 index=None, convert_influx_index=False,
+                                 use_old_csc_indexing=False):
         """Select a time series for a set of topics in a single subsystem
 
         Parameters
@@ -254,20 +276,33 @@ class EfdClient:
             `~astropy.time.TimeDelta`, compute a range centered on the start
             time (default is `False`).
         index : `int`, optional
-            For indexed topics set this to the index of the topic to query
+            When index is used, add an 'AND salIndex = index' to the query.
             (default is `None`).
+        convert_influx_index : `bool`, optional
+            Convert influxDB time index from TAI to UTC?  This is for using legacy
+            instances that may still have timestamps stored internally as TAI.
+            Modern instances all store index timestamps as UTC natively.
+            Default is `False`, don't translate from TAI to UTC.
+        use_old_csc_indexing: `bool`, optional
+            When index is used, add an 'AND {CSCName}ID = index' to the query
+            which is the old CSC indexing name.
+            (default is `False`).
 
         Returns
         -------
         result : `pandas.DataFrame`
             A `pandas.DataFrame` containing the results of the query.
         """
-        query = self.build_time_range_query(topic_name, fields, start, end, is_window, index)
+        query = self.build_time_range_query(topic_name, fields, start, end, is_window,
+                                            index, convert_influx_index,
+                                            use_old_csc_indexing)
         # Do query
-        ret = await self._do_query(query)
+        ret = await self._do_query(query, convert_influx_index)
         return ret
 
-    async def select_top_n(self, topic_name, fields, num, time_cut=None, index=None):
+    async def select_top_n(self, topic_name, fields, num, time_cut=None,
+                           index=None, convert_influx_index=False,
+                           use_old_csc_indexing=False):
         """Select the most recent N samples from a set of topics in a single subsystem.
         This method does not guarantee sort direction of the returned rows.
 
@@ -283,8 +318,17 @@ class EfdClient:
             Use a time cut instead of the most recent entry.
             (default is `None`)
         index : `int`, optional
-            For indexed topics set this to the index of the topic to query
-            (default is `None`)
+            When index is used, add an 'AND salIndex = index' to the query.
+            (default is `None`).
+        convert_influx_index : `bool`, optional
+            Convert influxDB time index from TAI to UTC?  This is for using legacy
+            instances that may still have timestamps stored internally as TAI.
+            Modern instances all store index timestamps as UTC natively.
+            Default is `False`, don't translate from TAI to UTC.
+        use_old_csc_indexing: `bool`, optional
+            When index is used, add an 'AND {CSCName}ID = index' to the query
+            which is the old CSC indexing name.
+            (default is `False`).
 
         Returns
         -------
@@ -300,9 +344,13 @@ class EfdClient:
         if time_cut:
             pstr = f" WHERE time < '{time_cut}Z'"
         if index:
-            parts = topic_name.split('.')
+            if use_old_csc_indexing:
+                parts = topic_name.split('.')
+                index_name = f'{parts[-2]}ID'  # The CSC name is always the penultimate
+            else:
+                index_name = 'salIndex'
             # The CSC name is always the penultimate
-            istr = f'{parts[-2]}ID = {index}'
+            istr = f'{index_name} = {index}'
             if pstr != '':
                 pstr += f' AND {istr}'
             else:
@@ -318,7 +366,7 @@ class EfdClient:
         query = f'SELECT {", ".join(fields)} FROM "{self.db_name}"."autogen"."{topic_name}"{pstr} {limit}'
 
         # Do query
-        ret = await self._do_query(query)
+        ret = await self._do_query(query, convert_influx_index)
         return ret
 
     def _make_fields(self, fields, base_fields):
@@ -359,7 +407,9 @@ class EfdClient:
         return ret, n
 
     async def select_packed_time_series(self, topic_name, base_fields, start, end,
-                                        is_window=False, index=None, ref_timestamp_col="cRIO_timestamp"):
+                                        is_window=False, index=None, ref_timestamp_col="cRIO_timestamp",
+                                        ref_timestamp_fmt='unix_tai', ref_timestamp_scale='tai',
+                                        convert_influx_index=False, use_old_csc_indexing=False):
         """Select fields that are time samples and unpack them into a dataframe.
 
         Parameters
@@ -380,11 +430,26 @@ class EfdClient:
             `~astropy.time.TimeDelta`, compute a range centered on the start
             time (default is `False`).
         index : `int`, optional
-            For indexed topics set this to the index of the topic to query
-            (default is `False`).
+            When index is used, add an 'AND salIndex = index' to the query.
+            (default is `None`).
         ref_timestamp_col : `str`, optional
             Name of the field name to use to assign timestamps to unpacked
             vector fields (default is 'cRIO_timestamp').
+        ref_timestamp_fmt : `str`, optional
+            Format to use to translating ``ref_timestamp_col`` values
+            (default is 'unix_tai').
+        ref_timestamp_scale : `str`, optional
+            Time scale to use in translating ``ref_timestamp_col`` values
+            (default is 'tai').
+        convert_influx_index : `bool`, optional
+            Convert influxDB time index from TAI to UTC?  This is for using legacy
+            instances that may still have timestamps stored internally as TAI.
+            Modern instances all store index timestamps as UTC natively.
+            Default is `False`, don't translate from TAI to UTC.
+        use_old_csc_indexing: `bool`, optional
+            When index is used, add an 'AND {CSCName}ID = index' to the query
+            which is the old CSC indexing name.
+            (default is `False`).
 
         Returns
         -------
@@ -401,11 +466,14 @@ class EfdClient:
         field_list = []
         for k in qfields:
             field_list += qfields[k]
-        result = await self.select_time_series(topic_name, field_list+[ref_timestamp_col, ],
-                                               start, end, is_window=is_window, index=index)
+        result = await self.select_time_series(topic_name, field_list + [ref_timestamp_col, ],
+                                               start, end, is_window=is_window, index=index,
+                                               convert_influx_index=convert_influx_index,
+                                               use_old_csc_indexing=use_old_csc_indexing)
         vals = {}
         for f in base_fields:
-            df = merge_packed_time_series(result, f, ref_timestamp_col=ref_timestamp_col)
+            df = merge_packed_time_series(result, f, ref_timestamp_col=ref_timestamp_col,
+                                          fmt=ref_timestamp_fmt, scale=ref_timestamp_scale)
             vals[f] = df[f]
         vals.update({'times': df['times']})
         return pd.DataFrame(vals, index=df.index)
@@ -444,13 +512,11 @@ class EfdClient:
                 vals['description'].append(None)
             if 'units' in f:
                 vals['units'].append(f['units'])
-                try:
-                    if vals['units'][-1] == 'unitless' or vals['units'][-1] == 'dimensionless':  # Special case not having units
-                        vals['aunits'].append(u.dimensionless_unscaled)
-                    else:
-                        vals['aunits'].append(u.Unit(vals['units'][-1]))
-                except (ValueError, TypeError) as e:
-                    logging.warning(f'Could not construct units: {e.args[0]}')
+                # Special case not having units
+                if vals['units'][-1] == 'unitless' or vals['units'][-1] == 'dimensionless':
+                    vals['aunits'].append(u.dimensionless_unscaled)
+                else:
+                    vals['aunits'].append(u.Unit(vals['units'][-1]))
             else:
                 vals['units'].append(None)
                 vals['aunits'].append(None)
