@@ -36,10 +36,12 @@ def auth_client():
 @pytest.fixture
 @pytest.mark.vcr
 async def efd_client():
-    df = pd.read_hdf(PATH/'efd_test.hdf')
+    df = pd.read_hdf(PATH / 'efd_test.hdf')
+    df1 = pd.read_hdf(PATH / 'efd_index_test.hdf', 'table')
     async with InfluxDBClient(db='client_test', mode='async', output='dataframe') as client:
         await client.create_database()
         await client.write(df, measurement='lsst.sal.fooSubSys.test')
+        await client.write(df1, measurement='lsst.sal.barSubSys.test')
         efd_client = EfdClient('test_efd', db_name='client_test', client=client)
         # Monkey patch the client to point to an existing schema registry
         # Note this is only available if on the NCSA VPN
@@ -51,7 +53,7 @@ async def efd_client():
 @pytest.fixture
 def expected_strs():
     expected = []
-    with open(PATH/'expected.txt', 'r') as fh:
+    with open(PATH / 'expected.txt', 'r') as fh:
         for line in fh.readlines():
             expected.append(line)
     return expected
@@ -59,17 +61,24 @@ def expected_strs():
 
 @pytest.fixture
 def test_df():
-    return pd.read_hdf(PATH/'efd_test.hdf')
+    return pd.read_hdf(PATH / 'efd_test.hdf')
+
 
 @pytest.fixture
 def test_query_res():
-    return pd.read_hdf(PATH/'packed_data.hdf', key='test_data')
+    return pd.read_hdf(PATH / 'packed_data.hdf', key='test_data')
+
 
 @pytest.fixture
 def start_stop():
     time = Time('2020-01-28T23:07:19.00', format='isot', scale='utc')
     return (time, time + TimeDelta(600, format='sec'))
 
+@pytest.fixture
+def start_stop_old():
+    time = Time('2020-01-27T23:07:19.00', format='isot', scale='utc')
+    return (time, time + TimeDelta(600, format='sec'))
+
 
 def test_bad_endpoint():
     with pytest.raises(IOError):
@@ -86,14 +95,19 @@ def test_auth_registry(auth_creds):
     assert auth_creds[1] == 'https://schema-registry-foo.bar.baz'
 
 
+@pytest.mark.vcr
+def test_auth_port(auth_creds):
+    assert auth_creds[2] == '443'
+
+
 @pytest.mark.vcr
 def test_auth_user(auth_creds):
-    assert auth_creds[2] == 'foo'
+    assert auth_creds[3] == 'foo'
 
 
 @pytest.mark.vcr
 def test_auth_password(auth_creds):
-    assert auth_creds[3] == 'bar'
+    assert auth_creds[4] == 'bar'
 
 
 @pytest.mark.vcr
@@ -122,6 +136,16 @@ def test_build_query(efd_client, start_stop, expected_strs):
     qstr = efd_client.build_time_range_query('lsst.sal.fooSubSys.test', ['foo', 'bar'],
                                              start_stop[0], start_stop[1])
     assert qstr == expected_strs[1].strip()
+    # Check old indexed component fetching works
+    qstr = efd_client.build_time_range_query('lsst.sal.fooSubSys.test', ['foo', 'bar'],
+                                             start_stop[0], start_stop[1],
+                                             index=2, use_old_csc_indexing=True)
+    assert qstr == expected_strs[4].strip()
+    # Check new indexed component fetching works
+    qstr = efd_client.build_time_range_query('lsst.sal.fooSubSys.test', ['foo', 'bar'],
+                                             start_stop[0], start_stop[1],
+                                             index=2)
+    assert qstr == expected_strs[5].strip()
 
 
 @pytest.mark.vcr
@@ -143,15 +167,15 @@ async def test_parse_schema(efd_client):
     # Body that we expect the registry API to return given the request.
     expected_body = {
         "schema": json.dumps(
-             {
+            {
                 "name": "schema1",
                 "type": "record",
                 "fields": [{"name": "a", "type": "int", "description": "Description 1", "units": "second"},
                            {"name": "b", "type": "double", "description": "Description 2", "units": "meter"},
                            {"name": "c", "type": "float", "description": "Description 3", "units": "gram"},
                            {"name": "d", "type": "string", "description": "Description 4", "units": "torr"}
-                          ],
-              }
+                           ],
+            }
         ),
         "subject": "schema1",
         "version": 1,
@@ -175,12 +199,39 @@ async def test_parse_schema(efd_client):
         assert isinstance(unit, u.UnitBase)
 
 
+@pytest.mark.asyncio
+async def test_bad_units(efd_client):
+    """Test that the EfdClient._parse_schema method raises when a bad astropy unit definition is passed."""
+    # Body that we expect the registry API to return given the request.
+    expected_body = {
+        "schema": json.dumps(
+            {
+                "name": "schema1",
+                "type": "record",
+                "fields": [{"name": "a", "type": "int", "description": "Description 1", "units": "not_unit"},
+                           ],
+            }
+        ),
+        "subject": "schema1",
+        "version": 1,
+        "id": 2,
+    }
+
+    body = json.dumps(expected_body).encode("utf-8")
+    client = MockRegistryApi(body=body)
+
+    schema = await client.get_schema_by_subject("schema1")
+    with pytest.raises(ValueError):
+        efd_client._parse_schema("schema1", schema)
+
+
 @pytest.mark.asyncio
 @pytest.mark.vcr
 async def test_topics(efd_client):
     topics = await efd_client.get_topics()
-    assert len(topics) == 1
-    assert topics[0] == 'lsst.sal.fooSubSys.test'
+    assert len(topics) == 2
+    assert topics[1] == 'lsst.sal.fooSubSys.test'
+    assert topics[0] == 'lsst.sal.barSubSys.test'
 
 
 @pytest.mark.asyncio
@@ -193,12 +244,40 @@ async def test_fields(efd_client, test_df):
 
 @pytest.mark.asyncio
 @pytest.mark.vcr
-async def test_time_series(efd_client, start_stop):
+async def test_time_series(efd_client, start_stop, start_stop_old):
     df = await efd_client.select_time_series('lsst.sal.fooSubSys.test', ['foo', 'bar'],
                                              start_stop[0], start_stop[1])
     assert len(df) == 600
     for c in ['foo', 'bar']:
         assert c in df.columns
+    df_legacy = await efd_client.select_time_series('lsst.sal.fooSubSys.test', ['foo', 'bar'],
+                                                    start_stop[0], start_stop[1], convert_influx_index=True)
+    # Test that df_legacy is in UTC assuming df was in TAI
+    t = Time(df.index).unix - Time(df_legacy.index).unix
+    # The indexes should all be the same since both the time range and index were shifted
+    assert numpy.all(t == 0.)
+    # But the queries should be different
+    assert not efd_client.query_history[-2] == efd_client.query_history[-1]
+    # Test indexed query
+    df1 = await efd_client.select_time_series('lsst.sal.barSubSys.test', ['eggs', 'ham'],
+                                              start_stop[0], start_stop[1], index=2)
+    assert len(df1) == 100
+    assert numpy.all(df1["eggs"] == 10)
+    # Old index should return similar sized dataframe
+    df1 = await efd_client.select_time_series('lsst.sal.barSubSys.test', ['eggs', 'ham'],
+                                              start_stop_old[0], start_stop_old[1], index=2,
+                                              use_old_csc_indexing=True)
+    assert len(df1) == 100
+    assert numpy.all(df1["eggs"] == 10)
+    # Using new indexing across old time frame should return empty dataframe
+    df1 = await efd_client.select_time_series('lsst.sal.barSubSys.test', ['eggs', 'ham'],
+                                              start_stop_old[0], start_stop_old[1], index=2)
+    assert len(df1) == 0
+    # Using old indexing across new time frame should return empty dataframe
+    df1 = await efd_client.select_time_series('lsst.sal.barSubSys.test', ['eggs', 'ham'],
+                                              start_stop[0], start_stop[1], index=2,
+                                              use_old_csc_indexing=True)
+    assert len(df1) == 0
 
 
 @pytest.mark.asyncio
@@ -208,6 +287,11 @@ async def test_top_n(efd_client, start_stop):
     assert len(df) == 10
     for c in ['foo', 'bar']:
         assert c in df.columns
+    df_legacy = await efd_client.select_top_n('lsst.sal.fooSubSys.test', ['foo', 'bar'],
+                                              10, convert_influx_index=True)
+    # Test that df_legacy is in UTC assuming df was in TAI
+    t = Time(df.index).unix - Time(df_legacy.index).unix
+    assert numpy.all(t == 37.)
     df = await efd_client.select_top_n('lsst.sal.fooSubSys.test', ['foo', 'bar'], 10, time_cut=start_stop[0])
     assert len(df) == 10
     for c in ['foo', 'bar']:
@@ -224,6 +308,9 @@ async def test_packed_time_series(efd_client, start_stop, test_query_res):
     df_exp = test_query_res
     df = await efd_client.select_packed_time_series('lsst.sal.fooSubSys.test', ['ham', 'egg', 'hamegg'],
                                                     start_stop[0], start_stop[1])
+    # The column 'times' holds the input to the packed time index.
+    # It's typically in TAI, but the returned index should be in UTC
+    assert numpy.all((numpy.array(df['times']) - Time(df.index).unix) == 37.)
     assert numpy.all((df.index[1:] - df.index[:-1]).total_seconds() > 0)
     assert numpy.all(df == df_exp)
     for c in ['ham', 'egg']:
@@ -235,7 +322,7 @@ def test_resample(test_query_res):
     df_copy = df.copy()
     df_copy.set_index(df_copy.index + pd.Timedelta(0.05, unit='s'), inplace=True)
     df_out = resample(df, df_copy)
-    assert len(df_out) == 2*len(df)
+    assert len(df_out) == 2 * len(df)
 
 
 def test_rendezvous(test_df):
